@@ -69,11 +69,18 @@ For full documentation and more advanced usage, please see
 """
 
 __all__ = [
+    # Base exports
     'JSONSerializable',
     'JSONWizard',
     'LoadMixin',
     'DumpMixin',
     'property_wizard',
+    # Helper serializer functions + meta config
+    'fromlist',
+    'fromdict',
+    'asdict',
+    'LoadMeta',
+    'DumpMeta',
     # Models
     'json_field',
     'json_key'
@@ -81,9 +88,10 @@ __all__ = [
 
 import logging
 
+from .bases_meta import LoadMeta, DumpMeta
 from .constants import PY36
-from .dumpers import DumpMixin, setup_default_dumper
-from .loaders import LoadMixin, setup_default_loader
+from .dumpers import DumpMixin, setup_default_dumper, asdict
+from .loaders import LoadMixin, setup_default_loader, fromlist, fromdict
 from .models import json_field, json_key
 from .property_wizard import property_wizard
 from .serial_json import JSONSerializable
