@@ -17,9 +17,6 @@ PY38_OR_ABOVE = sys.version_info[:2] >= (3, 8)
 # Check if currently running Python 3.10 or higher
 PY310_OR_ABOVE = sys.version_info[:2] >= (3, 10)
 
-# The dictionary key that identifies the default loader / dumper
-_DEFAULT = '__DEFAULT__'
-
 # The name of the dictionary object that contains `load` hooks for each
 # object type. Also used to check if a class is a :class:`BaseLoadHook`
 _LOAD_HOOKS = '__LOAD_HOOKS__'
@@ -35,3 +32,7 @@ SINGLE_ARG_ALIAS = '__SINGLE_ARG_ALIAS__'
 # Attribute name that will be defined for identity functions and methods;
 # mainly for internal use.
 IDENTITY = '__IDENTITY__'
+
+# The dictionary key that identifies the tag field for a class. This is only
+# set when the `tag` field is enabled in the `Meta` config for a dataclass.
+TAG = '__tag__'
