@@ -1,23 +1,25 @@
-from collections import defaultdict, deque
+from collections import defaultdict, deque, namedtuple
 from dataclasses import is_dataclass
 from datetime import datetime, time, date
 from decimal import Decimal
 from enum import Enum
 from typing import (
-    Any, Type, Dict, List, Tuple, Iterable, Sequence, Optional, Union,
-    NamedTupleMeta, SupportsFloat, SupportsInt, AnyStr, Text
+    Any, Type, Dict, List, Tuple, Iterable, Sequence, Union,
+    NamedTupleMeta, SupportsFloat, AnyStr, Text, Callable, Optional
 )
 from uuid import UUID
 
 from .abstractions import AbstractLoader, AbstractParser, FieldToParser
-from .bases import BaseLoadHook
+from .bases import BaseLoadHook, BaseMeta
 from .class_helper import (
     get_class_name, create_new_class,
     dataclass_to_loader, set_class_loader,
     dataclass_field_to_load_parser, json_field_to_dataclass_field,
+    _CLASS_TO_LOAD_FUNC, dataclass_fields, get_meta,
 )
-from .constants import _LOAD_HOOKS
-from .errors import ParseError
+from .constants import _LOAD_HOOKS, SINGLE_ARG_ALIAS, IDENTITY
+from .decorators import _alias, _single_arg_alias, resolve_alias_func, _identity
+from .errors import ParseError, MissingFields, UnknownJSONKey, MissingData
 from .log import LOG
 from .parsers import *
 from .type_def import (
@@ -25,12 +27,13 @@ from .type_def import (
     M, N, T, E, U, DD, LSQ, NT
 )
 from .utils.string_conv import to_snake_case
-from .utils.typing_compat import (
-    is_literal, is_typed_dict, get_origin, get_args, is_annotated
-)
 from .utils.type_conv import (
     as_bool, as_str, as_datetime, as_date, as_time, as_int
 )
+from .utils.typing_compat import (
+    is_literal, is_typed_dict, get_origin, get_args, is_annotated,
+    eval_forward_ref
+)
 
 
 class LoadMixin(AbstractLoader, BaseLoadHook):
@@ -52,14 +55,16 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
         setup_default_loader(cls)
 
     @staticmethod
+    @_alias(to_snake_case)
     def transform_json_field(string: str) -> str:
-
-        return to_snake_case(string)
+        # alias: to_snake_case
+        ...
 
     @staticmethod
-    def default_load_to(o: T, *_) -> T:
-
-        return o
+    @_identity
+    def default_load_to(o: T, _: Any) -> T:
+        # identity: o
+        ...
 
     @staticmethod
     def load_after_type_check(o: Any, base_type: Type[T]) -> T:
@@ -71,80 +76,86 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
         raise ParseError(e, o, base_type)
 
     @staticmethod
+    @_alias(as_str)
     def load_to_str(o: Union[Text, N, None], base_type: Type[str]) -> str:
-
-        return as_str(o, base_type)
+        # alias: as_str
+        ...
 
     @staticmethod
-    def load_to_int(o: Union[SupportsInt, str], base_type: Type[N]) -> N:
-
-        return as_int(o, base_type)
+    @_alias(as_int)
+    def load_to_int(o: Union[str, int, bool, None], base_type: Type[N]) -> N:
+        # alias: as_int
+        ...
 
     @staticmethod
+    @_single_arg_alias('base_type')
     def load_to_float(o: Union[SupportsFloat, str], base_type: Type[N]) -> N:
-
-        return base_type(o)
+        # alias: base_type(o)
+        ...
 
     @staticmethod
-    def load_to_bool(o: Union[str, bool, N], base_type: Type[bool]) -> bool:
-
-        return as_bool(o)
+    @_single_arg_alias(as_bool)
+    def load_to_bool(o: Union[str, bool, N], _: Type[bool]) -> bool:
+        # alias: as_bool(o)
+        ...
 
     @staticmethod
+    @_single_arg_alias('base_type')
     def load_to_enum(o: Union[AnyStr, N], base_type: Type[E]) -> E:
-
-        return base_type(o)
+        # alias: base_type(o)
+        ...
 
     @staticmethod
+    @_single_arg_alias('base_type')
     def load_to_uuid(o: Union[AnyStr, U], base_type: Type[U]) -> U:
-
-        return base_type(o)
+        # alias: base_type(o)
+        ...
 
     @staticmethod
     def load_to_iterable(
             o: Iterable, base_type: Type[LSQ],
             elem_parser: AbstractParser) -> LSQ:
 
-        return base_type(elem_parser(elem) for elem in o)
+        return base_type([elem_parser(elem) for elem in o])
 
-    @classmethod
+    @staticmethod
     def load_to_tuple(
-            cls, o: Union[List, Tuple], base_type: Type[Tuple],
+            o: Union[List, Tuple], base_type: Type[Tuple],
             elem_parsers: Sequence[AbstractParser]) -> Tuple:
 
-        if elem_parsers:
-            return base_type(parser(e) for parser, e in zip(elem_parsers, o))
+        try:
+            zipped = zip(elem_parsers, o)
+        except TypeError:
+            return base_type([e for e in o])
         else:
-            any_parser: AbstractParser = cls.get_parser_for_annotation(Any)
-            return base_type(any_parser(e) for e in o)
+            return base_type([parser(e) for parser, e in zipped])
 
-    @classmethod
+    @staticmethod
     def load_to_named_tuple(
-            cls, o: Union[Dict, List, Tuple], base_type: Type[NT],
-            field_to_parser: Optional[FieldToParser]) -> NT:
+            o: Union[Dict, List, Tuple], base_type: Type[NT],
+            field_to_parser: FieldToParser,
+            field_parsers: List[AbstractParser]) -> NT:
 
-        if field_to_parser is not None:
-            # Annotated as a sub-type of `typing.NamedTuple`
-            if isinstance(o, dict):
-                # Convert the values of all fields in the NamedTuple, using
-                # their type annotations. The keys in a dictionary object
-                # (assuming it was loaded from JSON) are required to be
-                # strings, so we don't need to convert them.
-                return base_type(**{k: field_to_parser[k](v)
-                                    for k, v in o.items()})
-            # We're passed in a list or a tuple.
-            field_parsers = list(field_to_parser.values())
-            return base_type(*[
-                parser(elem) for parser, elem in zip(field_parsers, o)])
+        if isinstance(o, dict):
+            # Convert the values of all fields in the NamedTuple, using
+            # their type annotations. The keys in a dictionary object
+            # (assuming it was loaded from JSON) are required to be
+            # strings, so we don't need to convert them.
+            return base_type(
+                **{k: field_to_parser[k](o[k]) for k in o})
+        # We're passed in a list or a tuple.
+        return base_type(
+            *[parser(elem) for parser, elem in zip(field_parsers, o)])
 
-        else:
-            # Annotated as just a regular `collections.namedtuple`
-            if isinstance(o, dict):
-                dict_parser = cls.get_parser_for_annotation(dict)
-                return base_type(**dict_parser(o))
-            # We're passed in a list or a tuple.
-            list_parser = cls.get_parser_for_annotation(list)
-            return base_type(*list_parser(o))
+    @staticmethod
+    def load_to_named_tuple_untyped(
+            o: Union[Dict, List, Tuple], base_type: Type[NT],
+            dict_parser: AbstractParser, list_parser: AbstractParser) -> NT:
+
+        if isinstance(o, dict):
+            return base_type(**dict_parser(o))
+        # We're passed in a list or a tuple.
+        return base_type(*list_parser(o))
 
     @staticmethod
     def load_to_dict(
@@ -196,20 +207,23 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
         return base_type(str(o))
 
     @staticmethod
+    @_alias(as_datetime)
     def load_to_datetime(
             o: Union[str, N], base_type: Type[datetime]) -> datetime:
-
-        return as_datetime(o, base_type)
+        # alias: as_datetime
+        ...
 
     @staticmethod
+    @_alias(as_time)
     def load_to_time(o: str, base_type: Type[time]) -> time:
-
-        return as_time(o, base_type)
+        # alias: as_time
+        ...
 
     @staticmethod
+    @_alias(as_date)
     def load_to_date(o: Union[str, N], base_type: Type[date]) -> date:
-
-        return as_date(o, base_type)
+        # alias: as_date
+        ...
 
     @classmethod
     def get_parser_for_annotation(cls, ann_type: Type[T],
@@ -248,7 +262,8 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
                 if isinstance(base_type, type):
 
                     if is_dataclass(base_type):
-                        load_hook = _load_to_dataclass
+                        base_type: Type[T]
+                        load_hook = load_func_for_dataclass(base_type)
 
                     elif issubclass(base_type, Enum):
                         load_hook = hooks.get(Enum)
@@ -258,11 +273,21 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
 
                     elif issubclass(base_type, tuple) \
                             and hasattr(base_type, '_fields'):
-                        load_hook = hooks.get(NamedTupleMeta)
-                        return NamedTupleParser(
-                            base_cls, base_type, load_hook,
-                            cls.get_parser_for_annotation
-                        )
+
+                        if hasattr(base_type, '__annotations__'):
+                            # Annotated as a `typing.NamedTuple` subtype
+                            load_hook = hooks.get(NamedTupleMeta)
+                            return NamedTupleParser(
+                                base_cls, base_type, load_hook,
+                                cls.get_parser_for_annotation
+                            )
+                        else:
+                            # Annotated as a `collections.namedtuple` subtype
+                            load_hook = hooks.get(namedtuple)
+                            return NamedTupleUntypedParser(
+                                base_cls, base_type, load_hook,
+                                cls.get_parser_for_annotation
+                            )
 
                     elif is_typed_dict(base_type):
                         load_hook = cls.load_to_typed_dict
@@ -275,9 +300,10 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
                     load_hook = cls.default_load_to
 
                 elif isinstance(base_type, (str, PyForwardRef)):
-                    return ForwardRefParser(
-                        base_cls, base_type, cls.get_parser_for_annotation
-                    )
+                    # Evaluate the forward reference here.
+                    base_type = eval_forward_ref(base_type, base_cls)
+
+                    return cls.get_parser_for_annotation(base_type, base_cls)
 
                 elif base_type is Ellipsis:
                     load_hook = cls.default_load_to
@@ -370,6 +396,28 @@ class LoadMixin(AbstractLoader, BaseLoadHook):
             return TupleParser(
                 base_cls, ann_type, load_hook, cls.get_parser_for_annotation)
 
+        if load_hook is None:
+            # If load hook is still not resolved at this point, it's possible
+            # the type is a subclass of a known type.
+            for typ in hooks:
+                if issubclass(base_type, typ):
+                    load_hook = hooks[typ]
+                    break
+            else:
+                # No matching hook is found for the type.
+                err = TypeError('Provided type is not currently supported.')
+                raise ParseError(
+                    err, None, base_type,
+                    unsupported_type=base_type
+                )
+
+        if hasattr(load_hook, SINGLE_ARG_ALIAS):
+            load_hook = resolve_alias_func(load_hook, locals())
+            return SingleArgParser(base_cls, base_type, load_hook)
+
+        if hasattr(load_hook, IDENTITY):
+            return IdentityParser(base_type, base_type)
+
         return Parser(base_cls, base_type, load_hook)
 
 
@@ -396,6 +444,8 @@ def setup_default_loader(cls=LoadMixin):
     cls.register_load_hook(deque, cls.load_to_iterable)
     cls.register_load_hook(list, cls.load_to_iterable)
     cls.register_load_hook(tuple, cls.load_to_tuple)
+    # noinspection PyTypeChecker
+    cls.register_load_hook(namedtuple, cls.load_to_named_tuple_untyped)
     cls.register_load_hook(NamedTupleMeta, cls.load_to_named_tuple)
     cls.register_load_hook(defaultdict, cls.load_to_defaultdict)
     cls.register_load_hook(dict, cls.load_to_dict)
@@ -432,86 +482,171 @@ def get_loader(class_or_instance=None, create=False) -> Type[LoadMixin]:
         return set_class_loader(class_or_instance, LoadMixin)
 
 
-# TODO move to :class:`LoadMixin` if possible
-def _load_to_dataclass(d: Dict[str, Any], cls: Type[T]) -> T:
+def fromdict(cls: Type[T],
+             d: Dict[str, Any],
+             config: Optional[BaseMeta] = None) -> T:
     """
     Converts a Python dictionary object to a dataclass instance.
 
     Iterates over each dataclass field recursively; lists, dicts, and nested
     dataclasses will likewise be initialized as expected.
 
+    `config` is an optional ``LoadMeta`` configuration to set up for the
+    dataclass. Here's a sample usage of this below::
+
+        >>> load_cfg = LoadMeta(MyClass, key_transform='CAMEL')
+        >>> fromdict(MyClass, {"myStr": "value"}, load_cfg)
+
     """
+    try:
+        load = _CLASS_TO_LOAD_FUNC[cls]
+    except KeyError:
+        load = load_func_for_dataclass(cls)
+
+    return load(d)
+
+
+def fromlist(cls: Type[T], list_of_dict: List[Dict[str, Any]]) -> List[T]:
+    """
+    Converts a Python list object to a list of dataclass instances.
+
+    Iterates over each dataclass field recursively; lists, dicts, and nested
+    dataclasses will likewise be initialized as expected.
+
+    """
+    try:
+        load = _CLASS_TO_LOAD_FUNC[cls]
+    except KeyError:
+        load = load_func_for_dataclass(cls)
+
+    return [load(d) for d in list_of_dict]
+
+
+def load_func_for_dataclass(cls: Type[T]) -> Callable[[Dict[str, Any]], T]:
+
     # Gets the loader for the class, or the default loader otherwise.
     cls_loader = get_loader(cls)
-    # Need to create a separate dictionary to copy over the constructor args,
-    # as we don't want to mutate the original dictionary object.
-    cls_kwargs = {}
+
+    # Get the meta config for the class, or the default config otherwise.
+    meta = get_meta(cls)
+
     # This contains a mapping of the original field name to the parser for its
-    # annotated type; the item lookup will be case-insensitive.
+    # annotated type; the item lookup *can* be case-insensitive.
     field_to_parser = dataclass_field_to_load_parser(cls_loader, cls)
-    # A cached mapping of keys in a JSON or dictionary object to the
+
+    # A cached mapping of each key in a JSON or dictionary object to the
     # resolved dataclass field name; useful so we don't need to do a case
     # transformation (via regex) each time.
     json_to_dataclass_field = json_field_to_dataclass_field(cls)
 
-    # Loop over the dictionary object
-    for json_field, json_val in d.items():
+    def cls_fromdict(o: Dict[str, Any], *_):
+        """
+        De-serialize a dictionary `o` to an instance of a dataclass `cls`.
+        """
 
-        # Get the resolved dataclass field name
-        field_name = json_to_dataclass_field.get(json_field)
-
-        if not field_name:
-            # On an initial run, or no resolved field name
-
-            if field_name is ExplicitNull:
-                continue
-
-            # Transform JSON field name (typically camel-cased) to the
-            # snake-cased variant which is convention in Python.
-            underscored_field = cls_loader.transform_json_field(json_field)
-
-            try:
-                # Do a case-insensitive lookup of the dataclass field, and
-                # cache the mapping so we have it for next time
-                field_name = field_to_parser.get_key(underscored_field)
-                json_to_dataclass_field[json_field] = field_name
-
-            except KeyError:
-                # Else, we see an unknown field in the dictionary object
-                json_to_dataclass_field[json_field] = ExplicitNull
-                LOG.warning(
-                    'JSON field %r missing from dataclass schema, '
-                    'class=%r, parsed field=%r',
-                    json_field, get_class_name(cls), underscored_field)
-                continue
+        # Need to create a separate dictionary to copy over the constructor
+        # args, as we don't want to mutate the original dictionary object.
+        cls_kwargs = {}
 
+        # This try-block is here in case the object `o` is None.
         try:
-            # Note: pass the original cased field to the class constructor;
-            # don't use the lowercase result from `transform_json_field`
-            cls_kwargs[field_name] = field_to_parser[field_name](json_val)
+            # Loop over the dictionary object
+            for json_key in o:
 
-        except ParseError as e:
-            # We run into a parsing error while loading the field value; Add
-            # additional info on the Exception object before re-raising it
-            #
-            # First confirm these values are not already set by an inner
-            # dataclass. If so, it likely makes it easier to debug the cause.
-            if not e.class_name:
-                e.class_name = cls.__qualname__
-                e.field_name = field_name
+                # Get the resolved dataclass field name
+                try:
+                    field_name = json_to_dataclass_field[json_key]
+                    # Exclude JSON keys that don't map to any fields.
+                    if field_name is ExplicitNull:
+                        continue
+
+                except KeyError:
+                    try:
+                        field_name = lookup_field_for_json_key(o, json_key)
+                    except LookupError:
+                        continue
+
+                try:
+                    # Note: pass the original cased field to the class
+                    # constructor; don't use the lowercase result from
+                    # `transform_json_field`
+                    cls_kwargs[field_name] = field_to_parser[field_name](
+                        o[json_key])
+
+                except ParseError as e:
+                    # We run into a parsing error while loading the field
+                    # value; Add additional info on the Exception object
+                    # before re-raising it.
+                    #
+                    # First confirm these values are not already set by an
+                    # inner dataclass. If so, it likely makes it easier to
+                    # debug the cause. Note that this should already be
+                    # handled by the `setter` methods.
+                    e.class_name = cls
+                    e.field_name = field_name
+                    raise
+
+        except TypeError:
+            # If the object `o` is None, then raise an error with the relevant
+            # info included. Else, just re-raise the error.
+            if o is None:
+                raise MissingData(cls) from None
             raise
 
-    obj = cls(**cls_kwargs)
-    return obj
+        # Now pass the arguments to the constructor method, and return the new
+        # dataclass instance. If there are any missing fields, we raise them
+        # here.
 
+        try:
+            return cls(**cls_kwargs)
 
-# noinspection SpellCheckingInspection
-def fromdict(cls: Type[T], d: Dict[str, Any]) -> T:
-    """
-    Converts a Python dictionary object to a dataclass instance.
+        except TypeError as e:
+            raise MissingFields(
+                e, o, cls, cls_kwargs, dataclass_fields(cls)
+            ) from None
 
-    Iterates over each dataclass field recursively; lists, dicts, and nested
-    dataclasses will likewise be initialized as expected.
+    def lookup_field_for_json_key(o: Dict[str, Any], json_field: str):
+        """
+        Determines the dataclass field which a JSON key should map to. Note
+        this only runs the initial time, i.e. the first time we encounter the
+        key in a JSON object.
 
-    """
-    return _load_to_dataclass(d, cls)
+        :raises LookupError: If there no resolved field name for the JSON key.
+        :raises UnknownJSONKey: If there is no resolved field name for the
+          JSON key, and`raise_on_unknown_json_key` is enabled in the Meta
+          config for the class.
+        """
+
+        # Transform JSON field name (typically camel-cased) to the
+        # snake-cased variant which is convention in Python.
+        transformed_field = cls_loader.transform_json_field(json_field)
+
+        try:
+            # Do a case-insensitive lookup of the dataclass field, and
+            # cache the mapping so we have it for next time
+            field_name = field_to_parser.get_key(transformed_field)
+            json_to_dataclass_field[json_field] = field_name
+
+        except KeyError:
+            # Else, we see an unknown field in the dictionary object
+            json_to_dataclass_field[json_field] = ExplicitNull
+            LOG.warning(
+                'JSON field %r missing from dataclass schema, '
+                'class=%r, parsed field=%r',
+                json_field, get_class_name(cls), transformed_field)
+
+            # Raise an error here (if needed)
+            if meta.raise_on_unknown_json_key:
+                cls_fields = dataclass_fields(cls)
+                e = UnknownJSONKey(json_field, o, cls, cls_fields)
+                raise e from None
+
+            raise LookupError
+
+        return field_name
+
+    # Save the load function for the class, so we don't need to run this logic
+    # each time.
+    _CLASS_TO_LOAD_FUNC[cls] = cls_fromdict
+
+    return cls_fromdict
