@@ -12,15 +12,15 @@ __all__ = [
     'is_generic',
     'is_base_generic',
     'is_annotated',
-    'get_type_hints_with_extras'
+    'eval_forward_ref'
 ]
 
+import sys
 import typing
 from collections.abc import Callable
-from functools import partial
 
 from ..constants import PY36, PY38, PY310_OR_ABOVE
-from ..type_def import PyLiteral, PyTypedDicts
+from ..type_def import PyLiteral, PyTypedDicts, PyForwardRef
 
 
 # TODO maybe move this to `type_def` if it makes sense
@@ -52,11 +52,9 @@ if not PY36:    # pragma: no cover
     )
 
     try:
-        from typing_extensions import get_type_hints, _AnnotatedAlias
+        from typing_extensions import _AnnotatedAlias
     except ImportError:
-        from typing import get_type_hints, _AnnotatedAlias
-
-    _get_type_hints_with_extras = partial(get_type_hints, include_extras=True)
+        from typing import _AnnotatedAlias
 
 
     def _is_annotated(cls):
@@ -138,7 +136,6 @@ else:   # pragma: no cover
         typing.GenericMeta,
     )
 
-    from typing import get_type_hints as _get_type_hints_with_extras
     from typing_extensions import AnnotatedMeta
 
 
@@ -295,11 +292,20 @@ def is_annotated(cls):
     return _is_annotated(cls)
 
 
-def get_type_hints_with_extras(obj, **kwargs):
+# Note: need to wrap the annotation for `base_type` with a forward ref,
+# because Python 3.6 complains.
+def eval_forward_ref(base_type: 'typing.Union[str, PyForwardRef]',
+                     cls: typing.Type):
+    """
+    Evaluate a forward reference using the class globals, and return the
+    underlying type reference.
     """
-    Generally the same as :func:`typing.get_type_hints`, but passes
-    ``extras=True`` by default, so that we can also retrieve the arguments
-    for :class:`typing.Annotated` types.
 
-    """
-    return _get_type_hints_with_extras(obj, **kwargs)
+    if isinstance(base_type, str):
+        base_type = PyForwardRef(base_type, is_argument=False)
+
+    # Evaluate the ForwardRef here
+    base_globals = sys.modules[cls.__module__].__dict__
+
+    # noinspection PyProtectedMember
+    return typing._eval_type(base_type, base_globals, None)
